// Copyright 2014 Miloš Vukov, BSD-style copyright and disclaimer apply
var Diploma;
(function (Diploma) {
    (function (Core) {
        /** Keys. */
        var Keys;
        (function (Keys) {
            Keys[Keys["Left"] = 65] = "Left";
            Keys[Keys["Right"] = 68] = "Right";
            Keys[Keys["Up"] = 87] = "Up";
            Keys[Keys["Down"] = 83] = "Down";
            Keys[Keys["Next"] = 32] = "Next";
        })(Keys || (Keys = {}));

        /** Calibration steps. */
        var CalibrationStep;
        (function (CalibrationStep) {
            CalibrationStep[CalibrationStep["TopLeftPoint"] = 1] = "TopLeftPoint";
            CalibrationStep[CalibrationStep["BottomLeftPoint"] = 2] = "BottomLeftPoint";
            CalibrationStep[CalibrationStep["BottomRightPoint"] = 3] = "BottomRightPoint";
            CalibrationStep[CalibrationStep["Finish"] = 4] = "Finish";
        })(CalibrationStep || (CalibrationStep = {}));

        /** Handle calibration (mm to px). */
        var Calibration = (function () {
            function Calibration() {
            }
            /** Show/hide calibration wizard. */
            Calibration.prototype.ToggleCalibration = function () {
                var _this = this;
                if (this.IsCalibrationVisible()) {
                    // Hide wizard.
                    this.RemoveCalibrationSurface();
                } else {
                    // Initalize and show wizard.
                    this._step = 1 /* TopLeftPoint */;
                    this._topLeftFingerPosition = undefined;
                    this._bottomLeftFingerPosition = undefined;
                    this._bottomRightFingerPosition = undefined;

                    // On space key move to next step.
                    this._debouncedNextStep = _.debounce(function (event) {
                        _this.NextStep(event);
                    }, 300, false);
                    $(window).keydown(function (event) {
                        return _this._debouncedNextStep(event);
                    });

                    // Listen to finger position.
                    Diploma.Core.EventAggregator.Subscribe(Diploma.Core.Topics.Motion_MovePointer_FingerPosition, function (fingerPosition) {
                        _this._lastFingerPosition = fingerPosition;
                    });

                    this.ShowCalibrationSurface();
                }
            };

            /** Check if wizard is visible. */
            Calibration.prototype.IsCalibrationVisible = function () {
                // Check if root element of wizard exsist.
                var div = $("#calibrationSurface");
                return (div.length);
            };

            /**
            * Perform step.
            * @param keyEvent JQuery key event.
            */
            Calibration.prototype.NextStep = function (keyEvent) {
                if (keyEvent.which == 32 /* Next */) {
                    switch (this._step) {
                        case 1 /* TopLeftPoint */:
                            this._topLeftFingerPosition = new Diploma.Core.Point3(this._lastFingerPosition);
                            this._step++;
                            $("#calibrationInfo").text("HOLD finger to BOTTOM LEFT calibration point and press SPACE key.");
                            break;
                        case 2 /* BottomLeftPoint */:
                            this._bottomLeftFingerPosition = new Diploma.Core.Point3(this._lastFingerPosition);
                            this._step++;
                            $("#calibrationInfo").text("HOLD finger to BOTTOM RIGHT calibration point and press SPACE key.");
                            break;
                        case 3 /* BottomRightPoint */:
                            this._bottomRightFingerPosition = new Diploma.Core.Point3(this._lastFingerPosition);
                            this.Calculate();
                            this._step++;
                            $("#calibrationInfo").html("Calibration is finished. Press a SPACE key to EXIT.</br></br>You can do a FINE calibration with keys A (left), D (right), W (up), S (down).");
                            break;
                        case 4 /* Finish */:
                            this.RemoveCalibrationSurface();
                            break;
                    }
                }
            };

            /** Calculate mm to px. */
            Calibration.prototype.Calculate = function () {
                // Get calibration points positions.
                var topLeft = $("#cpTopLeft").position();
                var bottomLeft = $("#cpBottomLeft").position();
                var bottomRight = $("#cpBottomRight").position();

                // Vertical.
                var px = (bottomLeft.top - topLeft.top);
                var mm = (this._topLeftFingerPosition.Y - this._bottomLeftFingerPosition.Y);
                var mmToPxY = px / mm;

                // Horizontal.
                var px = (bottomRight.left - bottomLeft.left);
                var mm = (this._bottomRightFingerPosition.X - this._bottomLeftFingerPosition.X);
                var mmToPxX = px / mm;

                this.SaveToStorage(mmToPxX, mmToPxY);
            };

            /**
            * Save mm-to-px to local storage.
            * @param mmToPxX Horizontal mm-to-px.
            * @param mmToPxY Vertical mm-to-px.
            */
            Calibration.prototype.SaveToStorage = function (mmToPxX, mmToPxY) {
                chrome.storage.local.set({ "mmToPxX": mmToPxX, "mmToPxY": mmToPxY }, function () {
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_Calibration_Saved, 1 /* All */);
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_Calibration_Saved, 0 /* Local */);
                });
            };

            /** Get or create wizard if does not exsists. */
            Calibration.prototype.ShowCalibrationSurface = function () {
                var div = $("#calibrationSurface");
                if (!div.length) {
                    var surfaceString = '' + '<div id="calibrationSurface">' + '<div id="cpTopLeft">&nbsp;</div>' + '<div id="cpBottomLeft">&nbsp;</div>' + '<div id="cpBottomRight">&nbsp;</div>' + '<div id="calibrationInfoPanel">' + '<h1>CALIBRATION</h1></br>' + '<h2><label id="calibrationInfo">HOLD finger to TOP LEFT calibration point and press SPACE key.</label></h2>' + '</div>' + '</div>';

                    $("body").append(surfaceString);
                }
                return div;
            };

            /** Remove calibration wizard */
            Calibration.prototype.RemoveCalibrationSurface = function () {
                var div = $("#calibrationSurface");
                if (div.length) {
                    div.remove();
                }
            };

            /** Get instance of calibration (singleton). */
            Calibration.Default = function () {
                if (!Calibration._defaultCalibration) {
                    Calibration._defaultCalibration = new Calibration();
                }
                return Calibration._defaultCalibration;
            };
            return Calibration;
        })();
        Core.Calibration = Calibration;

        /** Fine calibration. Align cursor with finger. */
        var FineCalibration = (function () {
            function FineCalibration() {
                this.DiffX = 0;
                this.DiffY = 0;
            }
            /** Initalitze fine calibration. */
            FineCalibration.prototype.Initialize = function () {
                var _this = this;
                this.LoadDiffs();
                this._debouncedSaveToStorage = _.debounce(function (diffX, diffY) {
                    _this.SaveToStorage(diffX, diffY);
                }, 100, false);

                // Listen to fine calibration keys.
                $(window).keydown(function (event) {
                    return _this.ChangeDiff(event);
                });

                // Load diffs if fine calibration was executed in another instance.
                Diploma.Core.EventAggregator.Subscribe(Diploma.Core.Topics.Core_FineCalibration_Saved, function () {
                    _this.LoadDiffs();
                });
            };

            /** Load diffs from local storage. */
            FineCalibration.prototype.LoadDiffs = function () {
                var _this = this;
                chrome.storage.local.get(["fineDiffX", "fineDiffY"], function (items) {
                    _this.DiffX = items.fineDiffX == undefined ? 0 : items.fineDiffX;
                    _this.DiffY = items.fineDiffY == undefined ? 0 : items.fineDiffY;

                    // Inform local pointer, that diffs where loaded from storage
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_FineCalibration_Updated, 0 /* Local */);
                });
            };

            /**
            * Change diff on key press.
            * @param keyEvent JQuery key event.
            */
            FineCalibration.prototype.ChangeDiff = function (keyEvent) {
                var changed = false;
                switch (keyEvent.which) {
                    case 65 /* Left */:
                        this.DiffX--;
                        changed = true;
                        break;
                    case 68 /* Right */:
                        this.DiffX++;
                        changed = true;
                        break;
                    case 87 /* Up */:
                        this.DiffY--;
                        changed = true;
                        break;
                    case 83 /* Down */:
                        this.DiffY++;
                        changed = true;
                        break;
                }
                if (changed) {
                    // Inform local content script pointer that fineDiff has changed.
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_FineCalibration_Updated, 0 /* Local */);
                    this._debouncedSaveToStorage(this.DiffX, this.DiffY);
                }
            };

            /** Save diffs to local storage. */
            FineCalibration.prototype.SaveToStorage = function (diffX, diffY) {
                chrome.storage.local.set({ "fineDiffX": diffX, "fineDiffY": diffY }, function () {
                    // Inform all instances of content scripts that calibration was performed and saved.
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_FineCalibration_Saved, 1 /* All */);
                });
            };

            /** Get instance of fine calibration (singleton). */
            FineCalibration.Default = function () {
                if (!FineCalibration._defaultFineCalibration) {
                    FineCalibration._defaultFineCalibration = new FineCalibration();
                }
                return FineCalibration._defaultFineCalibration;
            };
            return FineCalibration;
        })();
        Core.FineCalibration = FineCalibration;
    })(Diploma.Core || (Diploma.Core = {}));
    var Core = Diploma.Core;
})(Diploma || (Diploma = {}));
//# sourceMappingURL=Calibration.js.map
