// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Core
{
    /** Keys. */
    enum Keys
    {
        Left = 65,  // A
        Right = 68, // D
        Up = 87,    // W
        Down = 83,  // S
        Next = 32   // Space
    }

    /** Calibration steps. */
    enum CalibrationStep
    {
        TopLeftPoint = 1,       // 1. step.
        BottomLeftPoint = 2,    // 2. step.
        BottomRightPoint = 3,   // 3. step.
        Finish = 4              // End.
    }

    /** Handle calibration (mm to px). */
    export class Calibration
    {
        private _debouncedNextStep;

        private _step;
        private _lastFingerPosition; // LeapMotion point.

        private _topLeftFingerPosition: Point3;
        private _bottomLeftFingerPosition: Point3;
        private _bottomRightFingerPosition: Point3;

        /** Show/hide calibration wizard. */
        public ToggleCalibration()
        {
            if (this.IsCalibrationVisible())
            {
                // Hide wizard.
                this.RemoveCalibrationSurface();
            }
            else
            {
                // Initalize and show wizard.
                this._step = CalibrationStep.TopLeftPoint;
                this._topLeftFingerPosition = undefined;
                this._bottomLeftFingerPosition = undefined;
                this._bottomRightFingerPosition = undefined;
                
                // On space key move to next step.
                this._debouncedNextStep = _.debounce((event) => { this.NextStep(event) }, 300, false);
                $(window).keydown((event) => this._debouncedNextStep(event));
                
                // Listen to finger position.
                EventAggregator.Subscribe(Core.Topics.Motion_MovePointer_FingerPosition, (fingerPosition) =>
                {
                    this._lastFingerPosition = fingerPosition;
                });

                this.ShowCalibrationSurface();
            }
        }
        
        /** Check if wizard is visible. */
        private IsCalibrationVisible()
        {
            // Check if root element of wizard exsist.
            var div = $("#calibrationSurface");
            return (div.length);
        }

        /**
         * Perform step.
         * @param keyEvent JQuery key event.
         */
        private NextStep(keyEvent: JQueryKeyEventObject)
        {
            if (keyEvent.which == Keys.Next)
            {
                switch (this._step)
                {
                    case CalibrationStep.TopLeftPoint:
                        this._topLeftFingerPosition = new Point3(this._lastFingerPosition);
                        this._step++;
                        $("#calibrationInfo").text("HOLD finger to BOTTOM LEFT calibration point and press SPACE key.");
                        break;
                    case CalibrationStep.BottomLeftPoint:
                        this._bottomLeftFingerPosition = new Point3(this._lastFingerPosition);
                        this._step++;
                        $("#calibrationInfo").text("HOLD finger to BOTTOM RIGHT calibration point and press SPACE key.");
                        break;
                    case CalibrationStep.BottomRightPoint:
                        this._bottomRightFingerPosition = new Point3(this._lastFingerPosition);
                        this.Calculate();
                        this._step++;
                        $("#calibrationInfo").html("Calibration is finished. Press a SPACE key to EXIT.</br></br>You can do a FINE calibration with keys A (left), D (right), W (up), S (down).");
                        break;
                    case CalibrationStep.Finish:
                        this.RemoveCalibrationSurface();
                        break;
                }
            }
        }

        /** Calculate mm to px. */
        private Calculate()
        {
            // Get calibration points positions.
            var topLeft = $("#cpTopLeft").position();
            var bottomLeft = $("#cpBottomLeft").position();
            var bottomRight = $("#cpBottomRight").position();

            // Vertical.
            var px = (bottomLeft.top - topLeft.top);
            var mm = (this._topLeftFingerPosition.Y - this._bottomLeftFingerPosition.Y)
            var mmToPxY = px / mm;

            // Horizontal.
            var px = (bottomRight.left - bottomLeft.left);
            var mm = (this._bottomRightFingerPosition.X - this._bottomLeftFingerPosition.X)
            var mmToPxX = px / mm;

            this.SaveToStorage(mmToPxX, mmToPxY);
        }

        /**
         * Save mm-to-px to local storage.
         * @param mmToPxX Horizontal mm-to-px. 
         * @param mmToPxY Vertical mm-to-px.
         */
        private SaveToStorage(mmToPxX: number, mmToPxY: number)
        {
            chrome.storage.local.set({ "mmToPxX": mmToPxX, "mmToPxY": mmToPxY }, () =>
            {
                EventAggregator.Publish(Topics.Core_Calibration_Saved, PublishRange.All);
                EventAggregator.Publish(Topics.Core_Calibration_Saved, PublishRange.Local);
            });
        }

        /** Get or create wizard if does not exsists. */
        private ShowCalibrationSurface(): JQuery
        {
            var div = $("#calibrationSurface");
            if (!div.length)
            {
                var surfaceString = '' +
                    '<div id="calibrationSurface">' +
                    '<div id="cpTopLeft">&nbsp;</div>' +
                    '<div id="cpBottomLeft">&nbsp;</div>' +
                    '<div id="cpBottomRight">&nbsp;</div>' +
                    '<div id="calibrationInfoPanel">' +
                    '<h1>CALIBRATION</h1></br>' +
                    '<h2><label id="calibrationInfo">HOLD finger to TOP LEFT calibration point and press SPACE key.</label></h2>' +
                    '</div>' +
                    '</div>';

                $("body").append(surfaceString);
            }
            return div;
        }

        /** Remove calibration wizard */
        private RemoveCalibrationSurface()
        {
            var div = $("#calibrationSurface");
            if (div.length)
            {
                div.remove();
            }
        }

        private static _defaultCalibration;

        /** Get instance of calibration (singleton). */
        public static Default(): Calibration
        {
            if (!Calibration._defaultCalibration)
            {
                Calibration._defaultCalibration = new Calibration();
            }
            return Calibration._defaultCalibration;
        }
    }

    /** Fine calibration. Align cursor with finger. */
    export class FineCalibration
    {
        private _debouncedSaveToStorage;

        public DiffX: number = 0;
        public DiffY: number = 0;

        /** Initalitze fine calibration. */
        public Initialize()
        {
            this.LoadDiffs();
            this._debouncedSaveToStorage = _.debounce((diffX: number, diffY: number) =>
            {
                this.SaveToStorage(diffX, diffY);
            }, 100, false);
            // Listen to fine calibration keys.
            $(window).keydown((event) => this.ChangeDiff(event));
            // Load diffs if fine calibration was executed in another instance.
            EventAggregator.Subscribe(Topics.Core_FineCalibration_Saved, () =>
            {
                this.LoadDiffs();
            });
        }

        /** Load diffs from local storage. */
        private LoadDiffs()
        {
            chrome.storage.local.get(["fineDiffX", "fineDiffY"], (items: any) =>
            {
                this.DiffX = items.fineDiffX == undefined ? 0 : items.fineDiffX;
                this.DiffY = items.fineDiffY == undefined ? 0 : items.fineDiffY;
                // Inform local pointer, that diffs where loaded from storage
                EventAggregator.Publish(Topics.Core_FineCalibration_Updated, PublishRange.Local);
            });
        }

        /** 
         * Change diff on key press.
         * @param keyEvent JQuery key event.
         */
        private ChangeDiff(keyEvent: JQueryKeyEventObject)
        {
            var changed = false;
            switch (keyEvent.which)
            {
                case Keys.Left:
                    this.DiffX--;
                    changed = true;
                    break;
                case Keys.Right:
                    this.DiffX++;
                    changed = true;
                    break;
                case Keys.Up:
                    this.DiffY--;
                    changed = true;
                    break;
                case Keys.Down:
                    this.DiffY++;
                    changed = true;
                    break;
            }
            if (changed)
            {
                // Inform local content script pointer that fineDiff has changed.
                EventAggregator.Publish(Topics.Core_FineCalibration_Updated, PublishRange.Local);
                this._debouncedSaveToStorage(this.DiffX, this.DiffY);
            }
        }

        /** Save diffs to local storage. */
        private SaveToStorage(diffX: number, diffY: number)
        {
            chrome.storage.local.set({ "fineDiffX": diffX, "fineDiffY": diffY }, () =>
            {
                // Inform all instances of content scripts that calibration was performed and saved.
                EventAggregator.Publish(Topics.Core_FineCalibration_Saved, PublishRange.All);
            })
        }

        private static _defaultFineCalibration;

        /** Get instance of fine calibration (singleton). */
        public static Default(): FineCalibration
        {
            if (!FineCalibration._defaultFineCalibration)
            {
                FineCalibration._defaultFineCalibration = new FineCalibration();
            }
            return FineCalibration._defaultFineCalibration;
        }
    }
}