// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Core
{
    /** Event aggregator */
    export class EventAggregator
    {
        /** 
         * Publish message with data.
         * @param topic Message topic.
         * @param range Publish range.
         * @param data Message data. 
         */
        public static Publish(topic: string, range: PublishRange, data?: any): void
        {
            switch (range)
            {
                case PublishRange.All:
                    chrome.runtime.sendMessage({ Topic: topic, data: data });
                    break;
                case PublishRange.Local:
                    amplify.publish(topic, data);
                    break;
            }
        }

        /** 
         * Subscribe to message topic.
         * @param topic Message topic.
         * @param onMessage On message receive.
         */
        public static Subscribe(topic: string, onMessage: (any) => any): void
        {
            amplify.subscribe(topic, onMessage);
        }

        /** Initialize Event aggregator. */
        public static Initialize()
        {
            // Message from background script.
            chrome.runtime.onMessage.addListener(EventAggregator.OnServerPublishMessage);
        }

        /** Publish message local via amplifyjs. */
        private static OnServerPublishMessage(request, sender, sendResponse)
        {
            if (!request.Data) // Message does not have data.
                amplify.publish(request.Topic);
            else
                amplify.publish(request.Topic, request.Data);
            return true;
        }
    }

    /** Event aggregator hub. */
    export class EventAggregatorHub
    {
        private static _defaultHub;

        /** Initialize event aggregator hub. */
        Initialize()
        {
            // Message from content script.
            chrome.runtime.onMessage.addListener(this.OnClientPublishMessage)
        }

        /** Dispatch message to all content scripts */
        private OnClientPublishMessage(request, sender, sendResponse)
        {
            var fromTabId = sender.tab.id;
            // Get all tabs.
            chrome.tabs.query({}, (tabs) =>
            {
                for (var index = 0; index < tabs.length; index++)
                {
                    var id = tabs[index].id;
                    if (id != fromTabId)
                    {
                        // Send only to other tabs.
                        chrome.tabs.sendMessage(id, request);
                    }
                }
            });
            return true;
        }

        /** Get instance of event aggregator hub. */
        public static Default(): EventAggregatorHub
        {
            if (!EventAggregatorHub._defaultHub)
            {
                EventAggregatorHub._defaultHub = new EventAggregatorHub();
            }
            return EventAggregatorHub._defaultHub;
        }
    }

    /** Event aggregator publish range. */
    export enum PublishRange
    {
        Local, // Publish message local content script.
        All    // Publish message to all content scripts.
    }
}