// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Core
{
    /** Handle cursor */
    export class Pointer
    {
        private _pointerOffsetX = -20; // Pointer.Width/2
        private _pointerOffsetY = -20; // Pointer.Height/2
        
        private _currentType = CursorType.Pointer;

        /**
         * Move pointer.
         * @param x Pointer X position.
         * @param y Pointer Y position.
         */
        public MovePointer(x: number, y: number)
        {
            this._currentType = CursorType.Pointer;

            var highlight = this.GetHighlight();

            highlight.hide();

            // Move to current position
            var pointer = this.GetPointer();
            var css = {
                "left": (x + "px"),
                "top": (y + "px")
            };
            pointer.css(css);
            pointer.show();
            pointer.offset(pointer.offset()); // FIX: Causes repaint.
        }

        /** Animate screen tap on highlight cursor. */
        public ScreenTap()
        {
            if (this.IsCursor(CursorType.Highlight))
            {
                this.Using(() =>
                {
                    var highlight = this.GetHighlight();
                    highlight.one('webkitAnimationEnd', (event) =>
                    {
                        $(event.target).removeClass("pointer-animations-click");
                    });

                    // Play screentap animation
                    highlight.addClass("pointer-animations-click");
                    //highlight.height(highlight.height()); // TODO: Milos - Preveri oino deluje brez proanja (workarround) animacije

                }, CursorType.ScreenTap);
            }
        }

        private _lastHighlightedElement: JQuery;

        /**
         * Change cursor and highlight element.
         * @param element JQuery element to highlight.
         */
        public Highlight(element: JQuery)
        {
            if (!this.IsCursor(CursorType.Highlight))
            {
                this._currentType = CursorType.Highlight;

                // Set new type and hide current.
                this.GetPointer().hide();
                var highlight = this.GetHighlight();
                
                // Get element width and height.
                var elementPosition = element.offset();
                var h = element.innerHeight();
                var w = element.innerWidth();

                // Calculate highlight size.
                elementPosition.left = elementPosition.left - 3 - 3; // -3 for border and -(10/2) for h+10
                elementPosition.top = elementPosition.top - 3 - 3;
                highlight.offset(elementPosition);
                highlight.height(h + 8);
                highlight.width(w + 8);
                // Show highligt.
                highlight.show();
                highlight.offset(elementPosition);
                // Tell to navigator what was last highlighted element.
                EventAggregator.Publish(Topics.Core_Pointer_Highlight, PublishRange.Local, element.get(0));
                element.click();
            }
        }

        /** 
         * Check if current cursor is. 
         * @param cursorType Type of cursor.
         */
        private IsCursor(cursorType): boolean
        {
            var isCursor = this._currentType === cursorType;
            return isCursor;
        }

        /**
         * Set temporary cursor and revert back when finished.
         * @param execute Callback function to execute under temporary cursor.
         */
        private Using(execute: () => void, temporaryCursorType)
        {
            var currentType = this._currentType;
            try
            {
                this._currentType = temporaryCursorType;
                execute();
            }
            finally
            {
                this._currentType = currentType;
            }
        }

        /** Get pointer element. */
        private GetPointer(): JQuery
        {
            var div = this.GetDrawingSurface("body", "pointerSurface", (div) =>
            {
                var imageUrl = chrome.extension.getURL("/images/pointer40.png");
                var css = {
                    "background-image": "url(" + imageUrl + ")",
                    "z-index": ZIndex.Level1
                };
                div.css(css);
            });
            return div;
        }

        /** Get highlight element. */
        private GetHighlight(): JQuery
        {
            var div = this.GetDrawingSurface("body", "highlightSurface", (div) =>
            {
                var css = {
                    "z-index": ZIndex.Level1,
                    "border": "3px solid #0069cb",
                    "border-radius": "10px",
                    "box-shadow": "#B3B3B3 3px 3px 3px"
                };
                div.css(css);
            });
            return div;
        }

        /**  
         * Get or create drawing surface if does not exsists.
         * @param parentSelector Parent element of drawing surface.
         * @param surfaceId Unique surface id.
         * @param setup Additional function to setup drawing surface.
         */
        private GetDrawingSurface(parentSelector: string, surfaceId: string, setup: (div: JQuery) => void): JQuery
        {
            var div = $("#" + surfaceId);
            if (!div.length)
            {
                var css = {
                    "position": "fixed",
                    "width": "40px",
                    "height": "40px",
                    "display": "none",
                    "pointer-events": "none",               // FIX: getElementFromPoint
                    "background-position": "center center", // FIX: jQuery animation
                    "background-size": "100%",              // FIX: jQuery animation
                    "padding": "0px",
                    "margin": "0px"
                };

                $(parentSelector).append('<div id="' + surfaceId + '" />');
                div = $("#" + surfaceId);
                div.css(css);
                setup(div);
            }
            return div;
        }

        private static _defaultPointer;

        /** Get instance of pointer (singletone). */
        public static Default(): Pointer
        {
            if (!Pointer._defaultPointer)
            {
                Pointer._defaultPointer = new Pointer();
            }
            return Pointer._defaultPointer;
        }
    }

    /** Cursor types */
    enum CursorType
    {
        Pointer,    // Normal (circle).
        ScreenTap,  // Screen tap cursor (blink blue square).
        Highlight   // Highlight (blue square).
    }

    /** ZIndex */
    enum ZIndex
    {
        Level0 = 2147483647,    // On top.
        Level1 = 2147483646,    // On top - 1.
        Level2 = 2147483645     // On top - 2.
    }
}