// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Core
{
    /** Helper class for position and points */
    class PositionHelper
    {
        private _mmToPxX: number;
        private _mmToPxY: number;

        /** Initialize position helper. */
        constructor()
        {
            this.LoadFromStorage(); // Load mm-to-px from local storage.
            EventAggregator.Subscribe(Topics.Core_Calibration_Saved, () =>
            {
                // Load from storage when calibration was performed on other content script.
                this.LoadFromStorage();
            });
        }

        /** Load mm-to-px from local storage. */
        private LoadFromStorage()
        {
            chrome.storage.local.get(["mmToPxX", "mmToPxY"], (items: any) =>
            {
                this._mmToPxX = items.mmToPxX == undefined ? (378 / 100) : items.mmToPxX;
                this._mmToPxY = items.mmToPxY == undefined ? (378 / 100) : items.mmToPxY;
            });
        }

        /** 
         * Get document 2D point from LeapMotion 3D point. 
         * @param lmPointMm LeapMotion point.
         */
        public LeapMotionToDocumentPoint(lmPointMm: Point3): Point3
        {
            var dX = this.LMtoD_X(this.MilimeterToPixelX(lmPointMm.X))
            var dY = this.LMtoD_Y(this.MilimeterToPixelY(lmPointMm.Y));
            var dZ = this.MilimeterToPixelY(0);
            var documentPoint = new Point3([dX, dY, dZ]);
            return documentPoint;
        }

        /**  
         * Convert X mm to px.
         * @param milimeter X value in milimeter.
         */
        private MilimeterToPixelX(milimeter: number): number
        {
            // Debug
            //var mmToPx = 356 / 100;     // 1680*1050
            //var mmToPx = 580 / 100;     // 1920*1280
            //var mmToPx = 378 / 100;     // browser calculated
            //var mmToPx = 352 / 100;     // with calibration
            return milimeter * this._mmToPxX;
        }

        /**  
         * Convert Y mm to px.
         * @param milimeter Y value in milimeter.
         */
        private MilimeterToPixelY(milimeter: number): number
        {
            // Debug
            //var mmToPx = 356 / 100;     // 1680*1050
            //var mmToPx = 580 / 100;     // 1920*1280
            //var mmToPx = 378 / 100;     // browser calculated
            //var mmToPx = 336 / 100;     // with calibration
            return milimeter * this._mmToPxY;
        }

        /**
         * Translate LeapMotion X coordinate to document X coordinate.
         * @param lmPointPx LeapMotion X coordinate in px.
         */
        private LMtoD_X(lmPointPx): number
        {
            var LMtoS_X = lmPointPx + screen.width / 2;
            return LMtoS_X - window.screenLeft + FineCalibration.Default().DiffX;
        }

        /**
         * Translate LeapMotion Y coordinate to document Y coordinate.
         * @param lmPointPx LeapMotion Y coordinate in px.
         */
        private LMtoD_Y(lmPointPx): number
        {
            var LMtoS_Y = screen.height - lmPointPx;
            return LMtoS_Y - window.screenTop + FineCalibration.Default().DiffY;
        }

        private static _defaultPositionHelper;

        /** Get instance of position helper (singleton). */
        public static Default(): PositionHelper
        {
            if (!PositionHelper._defaultPositionHelper)
            {
                PositionHelper._defaultPositionHelper = new PositionHelper();
            }
            return PositionHelper._defaultPositionHelper;
        }
    }

    /** 3D point. */
    export class Point3
    {
        private _pointD: Point3;

        /** Initialize Point3 from leapjs 3D point. */
        constructor(point: Array<number>)
        {
            this.X = point[0];
            this.Y = point[1];
            this.Z = point[2];
        }

        public X: number;
        public Y: number;
        public Z: number;

        /** Get document point ( from LeapMotion point ). */
        public ToPointD(): Point3
        {
            if (!this._pointD)
            {
                this._pointD = PositionHelper.Default().LeapMotionToDocumentPoint(this);
            }
            return this._pointD;
        }
    }

    /** Normal/direction vector */
    export class NormalVector extends Point3
    {
        /** Vector pointing to left in xy plane. */
        public Left(): boolean
        {
            return this.InAngle(2.6, 4.0); // ~150-~230
        }

        public ToPointD(): Point3
        {
            throw new OperationNotSupportedError();
        }

        /**
         * Check if it is in angle.
         * @param from Angle in radian.
         * @param to Angle in radian.
         */
        private InAngle(from: number, to: number): boolean
        {
            var vec2Lib = Leap.glMatrix.vec2;
            var vec2Len = vec2Lib.length(vec2Lib.fromValues(this.X, this.Y));
            var angle = Math.acos(this.X / vec2Len);
            return angle >= from && angle <= to;
        }
    }
}