// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Motion
{
    import Core = Diploma.Core;

    /** Move pointer. */
    export class MovePointer implements IInitializer
    {
        private _throttledMovePointer;
        private _lastFingerPosition: Core.Point3;

        /** Initialize move pointer. */
        constructor()
        {
            // Reduce pointer position update, to remove pointer trail.
            this._throttledMovePointer = _.throttle((fingerPosition: Core.Point3) =>
            {
                this.MovePointer(fingerPosition);
            }, 50, { trailing: false });

            Core.EventAggregator.Subscribe(Core.Topics.Core_FineCalibration_Updated, () =>
            {
                if (this._lastFingerPosition)
                    this.MovePointer(this._lastFingerPosition);
            });
        }

        /**
         * Initialize move pointer.
         * @param controller Initialized LeapMotion controller.
         */
        public Initialize(controller: Controller): void
        {
            controller.on('animationFrame', (frame) => this.MoveAction(frame));
        }

        /**
         * Move pointer
         * @param frame LeapMotion frame.
         */
        private MoveAction(frame: Frame): void
        {
            if (this.AtLeastOneFinger(frame))
            {
                // Get position and move pointer
                var leapFingerPoint = this.GetPointerFinger(frame).stabilizedTipPosition;
                Core.EventAggregator.Publish(Core.Topics.Motion_MovePointer_FingerPosition, Core.PublishRange.Local, leapFingerPoint)
                var positionVector = new Core.Point3(leapFingerPoint);
                this._throttledMovePointer(positionVector);
            }
        }

        /**
         * Get fingert that is pointer.
         * @param frame LeapMotion frame.
         */
        private GetPointerFinger(frame: Frame): Finger
        {
            // LM detected only one finger.
            if (frame.fingers.length == 1)
                return frame.fingers[0];

            // Get finger that is closer to screen.
            var pointer = frame.fingers[0];
            var pointerZ = (new Core.Point3(pointer.stabilizedTipPosition)).Z;
            for (var index = 1; index < frame.fingers.length; index++)
            {
                var finger = frame.fingers[index];
                var fingerZ = (new Core.Point3(finger.stabilizedTipPosition)).Z;
                if (fingerZ < pointerZ)
                {
                    pointer = finger;
                    pointerZ = fingerZ;
                }
            }
            return pointer;
        }

        /**
         * Check if there is at least one finger.
         * @param frame LeapMotion frame.
         */
        private AtLeastOneFinger(frame: Frame): boolean
        {
            return (frame.fingers && frame.fingers.length > 0);
        }

        /**
         * Move pointer in document to finger position.
         * @param fingerPosition LeapMotion finger point.
         */
        private MovePointer(fingerPosition: Core.Point3)
        {
            this._lastFingerPosition = fingerPosition;

            // Get document point x and y 
            var x = Math.round(fingerPosition.ToPointD().X);
            var y = Math.round(fingerPosition.ToPointD().Y);

            // Debug
            // Get window size
            //var w = window.innerWidth;
            //var h = window.innerHeight;
            // Lock that you can't move cursor outside window
            //if (x < 0) x = 0;
            //if (y < 0) y = 0;
            //if (x > w) x = w;
            //if (y > h) y = h;

            // Get pointer
            var pointer = Core.Pointer.Default();
            // Set pointer
            var clickableElement = this.GetClickableElement(x, y);
            if (clickableElement == null)
            {
                pointer.MovePointer(x, y);
            }
            else
            {
                pointer.Highlight(clickableElement);
            }
        }

        /**
         * Get element from point if it is clickable (or null if it is not).
         * @param x X coordinate in HTML document.
         * @param y Y coordinate in HTML document.
         */
        private GetClickableElement(x: number, y: number): JQuery
        {
            var element = <HTMLBaseElement>document.elementFromPoint(x, y);
            if (element != null &&
                (element.href != undefined || element.onclick != undefined || $(element).css("pointer")=="cursor"))
            {
                return $(element);
            }
            return null;
        }
    }
}